@@ -61,6 +61,23 @@ class Client(object):
         else:
             raise RuntimeError('failed to connect to {} ({})'.format(service_url,resp.status_code))
 
+    # GENERIC GET/POST JSON
+
+    def get_json(self,method,url,json):
+        resp = self.session.get(url,json=json)
+        if resp.status_code == 200:
+            return resp.json()
+        else:
+            return {'success':False,'status_code':resp.status_code}
+
+    def post_json(self,url,json):
+        resp = self.session.post(url,json=json)
+        if resp.status_code == 200:
+            return resp.json()
+        else:
+            return {'success':False,'status_code':resp.status_code}
+
+
     # DESCRIPTORS
 
     def get_descriptors(self,substance_names,descriptor_set='all',include_properties=False):
@@ -117,13 +134,14 @@ class Client(object):
         else:
             return {'success':False,'status_code':resp.status_code}
 
-    def compute_descriptors(self,featurization,file_paths,structure_random_seed=None,return_files=False,structure_sample_size=100):
+    def compute_descriptors(self,featurization,source_data={},file_paths={},structure_random_seed=None,return_files=False,structure_sample_size=100):
         """Compute substance descriptors by providing substance source data.
 
-        The provided file paths are uploaded and processed by Aionics.
-        Descriptors are computed for each file.
-        Only the 'lithium_ionic' featurization is supported at this time. 
-        It is expected to work transparently for ICSD CIF files,
+        Descriptors are computed for each entry in source_data or file_paths,
+        depending on which featurization is selected.
+        The source_data values should be SMILES string (for 'rdkit' featurization).
+        The files in file_paths are uploaded to Aionics.
+        This is expected to work for ICSD CIF files,
         CIF files generated by pymatgen or VESTA, 
         or POSCAR files.
 
@@ -135,24 +153,32 @@ class Client(object):
         ----------
         featurization : str
             Specifies which featurization to apply. 
-            This should be 'lithium_ionic', 'anion_redox', or 'molecular'.
-            The 'molecular' and 'anion_redox' options are disabled by default 
-            (only specific instances of the application are allowed to use them).
+            This should be 'lithium_ionic', 'anion_redox', or 'rdkit'.
+            The 'anion_redox' option is disabled by default 
+            (only specific instances of the application are allowed to use it).
+        source_data : dict 
+            Dict of source data (SMILES strings) for 'rdkit' featurization.
+            The keys of this dict are used to index
+            the returned table of descriptors.
         file_paths : dict 
-            Dict of file paths containing substance data.
+            Dict of file paths containing substance data
+            for 'lithium_ionic' or 'anion_redox' featurization.
             Each path is opened and uploaded to Aionics.
             The keys of this dict are used to index
             the returned table of descriptors.
         structure_random_seed : int
             Integer seed used for the random number generator
             that is used to sample packing fractions 
-            and site occupancies.
+            and site occupancies
+            for 'lithium_ionic' or 'anion_redox' featurization.
         structure_sample_size : int
             Number of structures to sample 
-            for structures with fractionally occupied sites.
+            for structures with fractionally occupied sites,
+            for 'lithium_ionic' or 'anion_redox' featurization.
         return_files : bool
             If True, a zip files is returned containing 
-            descriptors as json and all sampled structures as POSCAR files.
+            descriptors as json and all sampled structures as POSCAR files,
+            for 'lithium_ionic' or 'anion_redox' featurization.
 
         Returns
         -------
@@ -166,17 +192,28 @@ class Client(object):
             data={'process_tag':'DESCRIPTORS'+uuid.uuid4().hex}
         )
         procid = pid_resp.json()['process_id']
-        resp = self.session.post(
-            self.service_url+'api/compute_descriptors', 
-            data={
-                'featurization':featurization,
-                'process_id':procid,
-                'structure_random_seed':structure_random_seed,
-                'structure_sample_size':structure_sample_size,
-                'return_files':return_files
-            },
-            files={nm:open(fp,'rb') for nm,fp in file_paths.items()}
-        )
+        if featurization in ['lithium_ionic','anion_redox']:
+            resp = self.session.post(
+                self.service_url+'api/compute_descriptors', 
+                data={
+                    'featurization':featurization,
+                    'process_id':procid,
+                    'structure_random_seed':structure_random_seed,
+                    'structure_sample_size':structure_sample_size,
+                    'return_files':return_files
+                },
+                files={nm:open(fp,'rb') for nm,fp in file_paths.items()}
+            )
+        elif featurizaton == 'rdkit':
+            resp = self.session.post(
+                self.service_url+'api/compute_descriptors', 
+                data={
+                    'featurization':featurization,
+                    'process_id':procid,
+                    'source_data':source_data
+                },
+            )
+
         if resp.status_code == 200:
             return resp.json()
         else:
@@ -320,6 +357,28 @@ class Client(object):
         else:
             return {'success':False,'status_code':resp.status_code}
 
+    def get_dataset_id(self,dataset_name):
+        """Get the id of a dataset by providing its name.
+
+        Parameters
+        ----------
+        dataset_name : str 
+            Name of the desired dataset. 
+
+        Returns
+        -------
+        response : dict 
+            Response data or error report. 
+        """
+        resp = self.session.get(
+            self.service_url+'api/get_dataset_id',
+            params={'dataset_name':dataset_name}
+        )
+        if resp.status_code == 200:
+            return resp.json()
+        else:
+            return {'success':False,'status_code':resp.status_code}
+
     def get_dataset(self,dataset_id):
         """Download a dataset from Aionics by providing its id.
 
@@ -441,6 +500,28 @@ class Client(object):
         else:
             return {'success':False,'status_code':resp.status_code}
 
+    def get_model_id(self,model_name):
+        """Get the id of a model by providing its name.
+
+        Parameters
+        ----------
+        model_name : str 
+            Name of the desired model. 
+
+        Returns
+        -------
+        response : dict 
+            Response data or error report. 
+        """
+        resp = self.session.get(
+            self.service_url+'api/get_model_id',
+            params={'model_name':model_name}
+        )
+        if resp.status_code == 200:
+            return resp.json()
+        else:
+            return {'success':False,'status_code':resp.status_code}
+
     def new_model(self,dataset_id,name,description=''):
         """Create a new model on Aionics.
 
@@ -1364,7 +1445,6 @@ class Client(object):
         else:
             return {'success':False,'status_code':resp.status_code}
 
-
     def pull_shared_substance(self,name):
         """Copy a substance from the shared to the private substance tables.
 
@@ -1393,11 +1473,35 @@ class Client(object):
         else:
             return {'success':False,'status_code':resp.status_code}
 
-    def add_substance(self,name,source='',other_names=[],descriptors={},properties={},documents={}):
+    def add_substance(self,name,source='',other_names=[],descriptors={},properties={},documents={},featurization={}):
         """Add a substance to the private substance tables.
 
-        See the documentation for 
-        apyonics.substances_client.SubstancesClient.add_substance().
+        Parameters
+        ----------
+        name : str
+            A name for the Substance.
+        source : str
+            A representation of the structure of the Substance,
+            sufficient for extracting descriptors.
+            For example, a molecule might be represented by a SMILES string,
+            and a crystalline solid might be represented by a CIF file.
+        other_names : list
+            Any number of additional names to assign to the Substance.
+            Substance names must be unique, so if any of these names
+            are already assigned to substances in the database,
+            this operation will fail.
+        descriptors : dict
+            Keys are descriptor names, and values are descriptor values.
+            Descriptors are expected to be real-valued.
+        properties : dict
+            Keys are property names, and values are property values.
+            Properties are expected to be real-valued.
+        documents : dict
+            Keys are document names, and values are documents.
+            Documents may be any sort of jsonable object.
+        featurization : dict
+            Keys are featurizations to attempt on the new substance.
+            Values are dicts of settings pertaining to the featurization.
         """
         resp = self.session.post(
             self.service_url+'api/add_substance', 
@@ -1407,7 +1511,8 @@ class Client(object):
                 'properties':properties,
                 'documents':documents,
                 'source':source,
-                'other_names':other_names
+                'other_names':other_names,
+                'featurization':featurization
             }
         )
         if resp.status_code == 200:
@@ -1416,11 +1521,41 @@ class Client(object):
         else:
             return {'success':False,'status_code':resp.status_code}
 
-    def update_substance(self,name,source='',other_names=[],descriptors={},properties={},documents={}):
+    def update_substance(self,name,source='',other_names=[],
+            descriptors={},properties={},documents={},
+            remove_other_descriptors=False,
+            remove_other_properties=False,
+            remove_other_documents=False
+        ):
         """Update substance data in the private substance tables.
 
-        See the documentation for 
-        apyonics.substances_client.SubstancesClient.update_substance().
+        Parameters
+        ----------
+        name : str
+            A name that is already assigned to the Substance of interest. 
+        source : str
+            A string representing of the structure of the Substance,
+            sufficient for extracting descriptors.
+        other_names : list
+            Any number of additional names to assign to the Substance.
+        descriptors : dict
+            Keys are descriptor names, and values are descriptor values.
+        properties : dict
+            Keys are property names, and values are property values.
+        documents : dict
+            Keys are document names, and values are documents.
+            Documents may be strings or anything json-like.
+        remove_other_descriptors : bool 
+            If True, descriptors other than those provided will be deleted. 
+        remove_other_properties : bool 
+            If True, properties other than those provided will be deleted. 
+        remove_other_documents : bool 
+            If True, documents other than those provided will be deleted. 
+
+        Returns
+        -------
+        response : dict
+            Dict containing status report and results.
         """
         resp = self.session.post(
             self.service_url+'api/update_substance', 
@@ -1430,7 +1565,10 @@ class Client(object):
                 'other_names':other_names,
                 'descriptors':descriptors,
                 'properties':properties,
-                'documents':documents
+                'documents':documents,
+                'remove_other_descriptors':remove_other_descriptors,
+                'remove_other_properties':remove_other_properties,
+                'remove_other_documents':remove_other_documents
             }
         )
         if resp.status_code == 200:
@@ -1459,8 +1597,87 @@ class Client(object):
             json={'name':name,'location':location}
         )
         if resp.status_code == 200:
-            respdata = {'success':True,'data':resp.json()}
-            return resp.json()
+            respdata = resp.json()
+            return respdata
+        else:
+            return {'success':False,'status_code':resp.status_code}
+
+    def get_substance_set(self,name):
+        """Get a substance set by name.
+
+        Parameters
+        ----------
+        name : str
+            Name of the substance set.
+
+        Returns
+        -------
+        resp : dict
+            Contains status report and, if successful, substance set data.
+        """
+        resp = self.session.get(
+            self.service_url+'api/substance_set', 
+            json={'name':name}
+        )
+        if resp.status_code == 200: 
+            respdata = resp.json()
+            return respdata
+        else:
+            return {'success':False,'status_code':resp.status_code}
+
+    def new_substance_set(self,name,substance_names=[]):
+        """Create a new substance set.
+
+        Parameters
+        ----------
+        name : str
+            Name to assign to the new substance set.
+        substance_names : list of str
+            Substance names to add to the set.
+
+        Returns
+        -------
+        resp : dict
+            Contains status report and, if successful, new substance set id
+        """
+        resp = self.session.post(
+            self.service_url+'api/new_substance_set', 
+            json={'name':name,'substance_names':substance_names}
+        )
+        if resp.status_code == 200: 
+            respdata = resp.json()
+            return respdata
+        else:
+            return {'success':False,'status_code':resp.status_code}
+
+    def edit_substance_set(self,name,names_to_add=[],names_to_remove=[]):
+        """Update an existing substance set.
+
+        Parameters
+        ----------
+        name : str
+            Name of the substance set to be updated.
+        names_to_add : list of str
+            Substance names to add to the set. 
+        names_to_remove : list of str
+            Substance names to remove from the set. 
+
+        Returns
+        -------
+        resp : dict
+            Contains status report and, if successful, new substance set id
+        """
+        resp = self.session.post(
+            self.service_url+'api/edit_substance_set', 
+            json={
+                'name':name,
+                'names_to_add':names_to_add,
+                'names_to_remove':names_to_remove
+            }
+        )
+        if resp.status_code == 200: 
+            respdata = resp.json()
+            return respdata
         else:
             return {'success':False,'status_code':resp.status_code}
 
@@ -1742,7 +1959,7 @@ class Client(object):
             procstat = self.session.get(self.service_url+'process_status',params={'process_id':process_id}).json()
         return procstat
 
-    def check_model_status(self,model_id,stat,wait=False,wait_interval=1.):
+    def check_model_status(self,model_id,stat='',wait=False,wait_interval=1.):
         """Check a model's status, and optionally wait for it.
 
         Parameters
@@ -1786,9 +2003,8 @@ class Client(object):
         mdlstat = self.session.get(
             self.service_url+'model_status',
             params={'model_id':model_id,'status_tag':stat}
-        )
+        ).json()
         if wait:
-            mdlstat = mdlstat.json()
             if not mdlstat['ready']:
                 time.sleep(wait_interval)
                 mdlstat = self.session.get(
@@ -1797,7 +2013,7 @@ class Client(object):
                 ).json()
         return mdlstat
 
-    def check_design_status(self,design_id,stat,wait=False,wait_interval=1.):
+    def check_design_status(self,design_id,stat='',wait=False,wait_interval=1.):
         """Check a design's status, and optionally wait for it.
 
         Parameters
